# -*- coding: utf-8 -*-
# @author: leesoar

"""errors"""

AnoleUserAgentError = type("AnoleUserAgentError", (Exception, ), {})
AnoleNameError = type("AnoleNameError", (Exception, ), {})
AnoleCrawlError = type("AnoleCrawlError", (Exception, ), {})



# -*- coding: utf-8 -*-
# @author: leesoar

import random
from threading import Lock

from anole import setting, error
from anole.tool import load_cached


class UserAgent(object):
    def __init__(self, path=setting.UA_CACHE_PATH):
        assert isinstance(path, str), \
            'path must be string'

        self.path = path
        self.data = {}
        self.browsers = {}

        self.load()

    def load(self):
        try:
            with self.load.lock:
                self.data = load_cached(self.path)
                self.browsers = self.data['browsers']
        except error.AnoleUserAgentError:
            setting.logger("Error occurred during fetching data, please check net status.")

    load.lock = Lock()

    def __getitem__(self, attr):
        return self.__getattr__(attr)

    def __getattr__(self, attr):
        attr = attr.lower()

        if attr == "random":
            browser = random.choice(list(self.browsers.keys()))
        else:
            browser = setting.SHORTCUTS.get(attr, attr)

        try:
            return random.choice(self.browsers[browser])
        except KeyError as e:
            raise error.AnoleUserAgentError(f"Browser `{browser}` is not support.") from e

    def fake(self, header=None, browser="random"):
        header = header or {}

        assert isinstance(header, dict), \
            'header must be dict'

        if "user-agent" not in [key.lower() for key in header.keys()]:
            header.update({"user-agent": eval(f"self.{browser}")})

        return header


class Name(object):
    def __init__(self, path=setting.NAME_CACHE_PATH):
        assert isinstance(path, str), \
            'path must be string'

        self.path = path
        self.data = {}
        self.surname = []
        self.name = []

        self.load()

    def load(self):
        try:
            with self.load.lock:
                self.data = load_cached(self.path, setting.NAME_CACHE_URL)
                self.name = self.data['names']['name']
                self.surname = self.data['names']['surname']
        except error.AnoleNameError:
            setting.logger("Error occurred during fetching data, please check net status.")

    load.lock = Lock()

    def fake(self, length=None):
        if length:
            return random.choice(self.surname) + "".join(random.choices(self.name, k=length))
        return random.choice(self.surname) + "".join(random.choices(self.name, k=random.choice([1, 2])))



# -*- coding: utf-8 -*-
# @author: leesoar

"""tools"""

import json
import os

import requests
from retrying import retry

from anole import setting, error


@retry(stop_max_attempt_number=setting.HTTP_RETRY)
def get(url):
    res = requests.get(url)
    if res.status_code != 200:
        raise error.AnoleCrawlError("Download cache failed, please check.")
    return res


def update(path, data):
    os.path.exists(path) and os.remove(path)
    open(path, encoding="utf-8", mode="w").write(data)


def read(path):
    return json.loads(open(path, encoding="utf-8").read())


def load_cached(path, url=setting.UA_CACHE_URL):
    if not os.path.exists(path):
        update(path, get(url).text)
    return read(path)
